import os
from typing import List, Optional

import pystac
from stactools.core.io.xml import XmlElement

from .constants import (SAFE_MANIFEST_ASSET_KEY, SENTINEL_OLCI_BANDS,
                        SENTINEL_SLSTR_BANDS, SENTINEL_SRAL_BANDS,
                        SENTINEL_SYNERGY_BANDS)


class ManifestError(Exception):
    pass


class MetadataLinks:
    def __init__(
        self,
        granule_href: str,
    ):
        self.granule_href = granule_href
        self.href = os.path.join(granule_href, "xfdumanifest.xml")

        root = XmlElement.from_file(self.href)
        data_object_section = root.find("dataObjectSection")
        if data_object_section is None:
            raise ManifestError(
                f"Manifest at {self.href} does not have a dataObjectSection")

        self._data_object_section = data_object_section
        self.product_metadata_href = os.path.join(granule_href,
                                                  "xfdumanifest.xml")

    def _find_href(self, xpaths: List[str]) -> Optional[str]:
        file_path = None
        for xpath in xpaths:
            file_path = self._data_object_section.find_attr("href", xpath)
            if file_path is not None:
                break

        if file_path is None:
            return None
        else:
            # Remove relative prefix that some paths have
            file_path = file_path.strip("./")
            return os.path.join(self.granule_href, file_path)

    @property
    def thumbnail_href(self) -> Optional[str]:
        preview = os.path.join(self.granule_href, "preview")
        return os.path.join(preview, "quick-look.png")

    def create_manifest_asset(self):
        asset = pystac.Asset(
            href=self.href,
            media_type=pystac.MediaType.XML,
            roles=["metadata"],
        )
        return (SAFE_MANIFEST_ASSET_KEY, asset)

    def create_band_asset(self):

        band_dict_list = []

        root = XmlElement.from_file(self.product_metadata_href)
        product_type = root.findall(".//sentinel3:productType")[0].text

        if product_type.split("_")[0] == "OL":
            instrument_bands = SENTINEL_OLCI_BANDS
        elif product_type.split("_")[0] == "SL":
            instrument_bands = SENTINEL_SLSTR_BANDS
        elif product_type.split("_")[0] == "SR":
            instrument_bands = SENTINEL_SRAL_BANDS
        elif product_type.split("_")[0] == "SY":
            instrument_bands = SENTINEL_SYNERGY_BANDS

        if instrument_bands == SENTINEL_SRAL_BANDS:
            for band in instrument_bands:
                band_dict = {
                    "name": instrument_bands[band].name,
                    "description": instrument_bands[band].description,
                    "central_frequency":
                    instrument_bands[band].center_wavelength,
                    "band_width_in_Hz":
                    instrument_bands[band].full_width_half_max
                }
                band_dict_list.append(band_dict)
        elif instrument_bands == SENTINEL_SYNERGY_BANDS:
            if "AOD" in product_type:
                key_list = list(SENTINEL_SYNERGY_BANDS.keys())[26:32]
            elif "SYN" in product_type:
                key_list = list(SENTINEL_SYNERGY_BANDS.keys())[:26]
            else:
                key_list = list(SENTINEL_SYNERGY_BANDS.keys())[-4:]
            for band in key_list:
                band_dict = {
                    "name": instrument_bands[band].name,
                    "description": instrument_bands[band].description,
                    "center_wavelength":
                    instrument_bands[band].center_wavelength,
                    "band_width": instrument_bands[band].full_width_half_max
                }
                band_dict_list.append(band_dict)
        else:
            for band in instrument_bands:
                band_dict = {
                    "name": instrument_bands[band].name,
                    "description": instrument_bands[band].description,
                    "center_wavelength":
                    instrument_bands[band].center_wavelength,
                    "band_width": instrument_bands[band].full_width_half_max
                }
                band_dict_list.append(band_dict)

        asset = pystac.Asset(href=self.href,
                             media_type=pystac.MediaType.XML,
                             roles=["metadata"],
                             extra_fields={"band_fields": band_dict_list})
        return ("eo:bands", asset)



import logging

import pystac
from pystac.extensions.eo import EOExtension
from pystac.extensions.projection import ProjectionExtension
from pystac.extensions.sat import SatExtension

from .constants import SENTINEL_CONSTELLATION, SENTINEL_PROVIDER
from .metadata_links import MetadataLinks
from .product_metadata import ProductMetadata
from .properties import (fill_eo_properties, fill_proj_properties,
                         fill_sat_properties)

logger = logging.getLogger(__name__)


def create_item(granule_href: str) -> pystac.Item:
    """Create a STC Item from a Sentinel-3 scene.

    Args:
        granule_href (str): The HREF to the granule.
            This is expected to be a path to a SEN3 archive.

    Returns:
        pystac.Item: An item representing the Sentinel-3 OLCI or SLSTR scene.
    """

    metalinks = MetadataLinks(granule_href)

    product_metadata = ProductMetadata(metalinks.product_metadata_href)

    item = pystac.Item(
        id=product_metadata.scene_id,
        geometry=product_metadata.geometry,
        bbox=product_metadata.bbox,
        datetime=product_metadata.get_datetime,
        properties={},
        stac_extensions=[],
    )

    # ---- Add Extensions ----
    # sat
    sat = SatExtension.ext(item, add_if_missing=True)
    fill_sat_properties(sat, metalinks.product_metadata_href)

    # eo
    eo = EOExtension.ext(item, add_if_missing=True)
    fill_eo_properties(eo, metalinks.product_metadata_href)

    # proj
    proj = ProjectionExtension.ext(item, add_if_missing=True)
    fill_proj_properties(proj, product_metadata)

    # s3 properties
    item.properties.update({**product_metadata.metadata_dict})

    # --Common metadata--
    item.common_metadata.providers = [SENTINEL_PROVIDER]
    item.common_metadata.platform = product_metadata.platform
    item.common_metadata.constellation = SENTINEL_CONSTELLATION

    # Add assets to item
    item.add_asset(*metalinks.create_manifest_asset())

    # objects for bands
    item.add_asset(*metalinks.create_band_asset())

    return item



import os
from datetime import datetime
from typing import Any, Dict, List, Optional

from pystac.utils import str_to_datetime
from shapely.geometry import Polygon, mapping  # type: ignore
from stactools.core.io.xml import XmlElement


class ProductMetadataError(Exception):
    pass


class ProductMetadata:
    def __init__(
        self,
        href,
    ) -> None:
        self.href = href
        self._root = XmlElement.from_file(href)

        def _get_geometries():
            # Find the footprint descriptor
            footprint_text = self._root.findall(".//gml:posList")
            if footprint_text is None:
                ProductMetadataError(
                    f"Cannot parse footprint from product metadata at {self.href}"
                )
            # Convert to values
            footprint_value = [
                float(x)
                for x in footprint_text[0].text.replace(" ", ",").split(",")
            ]

            footprint_points = [
                p[::-1] for p in list(zip(*[iter(footprint_value)] * 2))
            ]

            footprint_polygon = Polygon(footprint_points)
            geometry = mapping(footprint_polygon)
            bbox = list(footprint_polygon.bounds)

            return (bbox, geometry)

        self.bbox, self.geometry = _get_geometries()

    @property
    def scene_id(self) -> str:
        """Returns the string to be used for a STAC Item id.

        Removes the processing number and .SAFE extension
        from the product_id defined below.

        Parsed based on the naming convention found here:
        https://sentinel.esa.int/web/sentinel/user-guides/sentinel-3-slstr/naming-convention
        """
        product_id = self.product_id
        # Ensure the product id is as expected.
        if not product_id.endswith(".SEN3"):
            raise ValueError("Unexpected value found at "
                             f"{product_id}: "
                             "this was expected to follow the sentinel 3 "
                             "naming convention, including "
                             "ending in .SEN3")

        scene_id = self.product_id.split(".")[0]

        return scene_id

    @property
    def product_id(self) -> str:
        # Parse the name from href as it doesn't exist in xml files
        href = self.href
        result = href.split("/")[-2]
        if result is None:
            raise ValueError(
                "Cannot determine product ID using product metadata "
                f"at {self.href}")
        else:
            return result

    @property
    def get_datetime(self) -> datetime:
        start_time = self._root.findall(".//sentinel-safe:startTime")[0].text
        end_time = self._root.findall(".//sentinel-safe:stopTime")[0].text

        central_time = (
            datetime.strptime(start_time, "%Y-%m-%dT%H:%M:%S.%fZ") +
            (datetime.strptime(end_time, "%Y-%m-%dT%H:%M:%S.%fZ") -
             datetime.strptime(start_time, "%Y-%m-%dT%H:%M:%S.%fZ")) / 2)

        if central_time is None:
            raise ValueError(
                "Cannot determine product start time using product metadata "
                f"at {self.href}")
        else:
            return str_to_datetime(str(central_time))

    @property
    def start_datetime(self) -> datetime:
        time = self._root.findall(".//sentinel-safe:startTime")

        if time is None:
            raise ValueError(
                "Cannot determine product start time using product metadata "
                f"at {self.href}")
        else:
            return str_to_datetime(time[0].text)

    @property
    def end_datetime(self) -> datetime:
        time = self._root.findall(".//sentinel-safe:stopTime")

        if time is None:
            raise ValueError(
                "Cannot determine product start time using product metadata "
                f"at {self.href}")
        else:
            return str_to_datetime(time[0].text)

    @property
    def platform(self) -> Optional[str]:

        family_name = self._root.findall(".//sentinel-safe:familyName")[0].text
        platform_name = self._root.findall(".//sentinel-safe:number")[0].text

        return family_name + platform_name

    @property
    def cycle_number(self) -> Optional[str]:

        return self._root.findall(".//safe:cycleNumber")[0].text

    @property
    def image_paths(self) -> List[str]:
        head_folder = os.path.dirname(self.href)
        measurements = os.path.join(head_folder, "measurement")
        return [x for x in os.listdir(measurements) if x.endswith("tiff")]

    @property
    def metadata_dict(self) -> Dict[str, Any]:
        product_type = self._root.findall(".//sentinel3:productType")[0].text
        if (product_type.split("_")[0] == "OL"
                and product_type.split("_")[1] == "1"):
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:salineWaterPixels"))),
                "s3:coastalPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:coastalPixels"))),
                "s3:freshInlandWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//sentinel3:freshInlandWaterPixels"))),
                "s3:tidalRegionPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:tidalRegionPixels"))),
                "s3:brightPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:brightPixels"))),
                "s3:invalidPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:invalidPixels"))),
                "s3:cosmeticPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:cosmeticPixels"))),
                "s3:duplicatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:duplicatedPixels"))),
                "s3:saturatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:saturatedPixels"))),
                "s3:dubiousSamples_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:dubiousSamples"))),
            }
        elif (product_type.split("_")[0] == "OL"
              and product_type.split("_")[1] == "2"):
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:salineWaterPixels"))),
                "s3:coastalPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:coastalPixels"))),
                "s3:freshInlandWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//sentinel3:freshInlandWaterPixels"))),
                "s3:tidalRegionPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:tidalRegionPixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
                "s3:invalidPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:invalidPixels"))),
                "s3:cosmeticPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:cosmeticPixels"))),
                "s3:duplicatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:duplicatedPixels"))),
                "s3:saturatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:saturatedPixels"))),
                "s3:dubiousSamples_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//olci:dubiousSamples"))),
            }
        elif (product_type.split("_")[0] == "SL"
              and product_type.split("_")[1] == "1"):
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:classificationSummary[@grid='1 km']"
                            "/sentinel3:salineWaterPixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:classificationSummary[@grid='1 km']"
                            "/sentinel3:landPixels"))),
                "s3:coastalPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:classificationSummary[@grid='1 km']"
                            "/sentinel3:coastalPixels"))),
                "s3:freshInlandWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:classificationSummary[@grid='1 km']"
                            "/sentinel3:freshInlandWaterPixels"))),
                "s3:tidalRegionPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:classificationSummary[@grid='1 km']"
                            "/sentinel3:tidalRegionPixels"))),
                "s3:cosmeticPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:pixelQualitySummary[@grid='1 km']"
                            "/slstr:cosmeticPixels"))),
                "s3:duplicatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:pixelQualitySummary[@grid='1 km']"
                            "/slstr:duplicatedPixels"))),
                "s3:saturatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:pixelQualitySummary[@grid='1 km']"
                            "/slstr:saturatedPixels"))),
                "s3:outOfRangePixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//slstr:pixelQualitySummary[@grid='1 km']"
                            "/slstr:outOfRangePixels"))),
            }
        elif (product_type.split("_")[0] == "SL"
              and product_type.split("_")[1] == "2"):
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:salineWaterPixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
                "s3:coastalPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:coastalPixels"))),
                "s3:freshInlandWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//sentinel3:freshInlandWaterPixels"))),
                "s3:tidalRegionPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:tidalRegionPixels"))),
                "s3:cosmeticPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//slstr:cosmeticPixels"))),
                "s3:duplicatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//slstr:duplicatedPixels"))),
                "s3:saturatedPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//slstr:saturatedPixels"))),
                "s3:outOfRangePixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//slstr:outOfRangePixels"))),
            }
        elif (product_type.split("_")[0] == "SR"
              and product_type.split("_")[1] == "2"):
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:lrmModePercentage":
                float(self._root.findall(".//sral:lrmModePercentage")[0].text),
                "s3:sarModePercentage":
                float(self._root.findall(".//sral:sarModePercentage")[0].text),
                "s3:landPercentage":
                float(self._root.findall(".//sral:landPercentage")[0].text),
                "s3:closedSeaPercentage":
                float(
                    self._root.findall(".//sral:closedSeaPercentage")[0].text),
                "s3:continentalIcePercentage":
                float(
                    self._root.findall(".//sral:continentalIcePercentage")
                    [0].text),
                "s3:openOceanPercentage":
                float(
                    self._root.findall(".//sral:openOceanPercentage")[0].text),
            }
        elif "SY_2_AOD" in product_type:
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:salineWaterPixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
            }
        elif "SY_2_SYN" in product_type:
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:salineWaterPixels"))),
                "s3:coastalPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:coastalPixels"))),
                "s3:freshInlandWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//sentinel3:freshInlandWaterPixels"))),
                "s3:tidalRegionPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:tidalRegionPixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
            }
        elif "SY_2_V10" in product_type:
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:snowOrIcePixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:snowOrIcePixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
            }
        elif "SY_2_VG1" in product_type:
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:snowOrIcePixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:snowOrIcePixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
            }
        elif "SY_2_VGP" in product_type:
            result = {
                "start_datetime":
                str(self.start_datetime),
                "end_datetime":
                str(self.end_datetime),
                "s3:instrument":
                str(
                    self._root.find_attr(
                        "abbreviation",
                        ".//sentinel-safe:familyName[@abbreviation]")),
                "s3:mode":
                str(self._root.find_attr("identifier",
                                         ".//sentinel-safe:mode")),
                "s3:productType":
                self._root.findall(".//sentinel3:productType")[0].text,
                "s3:snowOrIcePixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:snowOrIcePixels"))),
                "s3:salineWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:salineWaterPixels"))),
                "s3:coastalPixelss_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:coastalPixels"))),
                "s3:freshInlandWaterPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage",
                            ".//sentinel3:freshInlandWaterPixels"))),
                "s3:tidalRegionPixels_percentage":
                float(
                    str(
                        self._root.find_attr(
                            "percentage", ".//sentinel3:tidalRegionPixels"))),
                "s3:landPixels_percentage":
                float(
                    str(
                        self._root.find_attr("percentage",
                                             ".//sentinel3:landPixels"))),
            }
        return {k: v for k, v in result.items() if v is not None}

    @property
    def get_shape(self):
        x_size = int(self._root.findall(".//sentinel3:columns")[0].text)
        y_size = int(self._root.findall(".//sentinel3:rows")[0].text)
        shape = [x_size, y_size]

        return shape

    @property
    def get_epsg(self):
        epsg = self._root.find_attr(
            "srsName", ".//sentinel-safe:footPrint").split("/")[-1]

        return int(epsg)



import logging
import os

import click

from stactools.sentinel3.stac import create_item

logger = logging.getLogger(__name__)


def create_sentinel3_command(cli):
    """Creates the stactools-sentinel3 command line utility."""
    @cli.group(
        "sentinel3",
        short_help=("Commands for working with stactools-sentinel3"),
    )
    def sentinel3():
        pass

    @sentinel3.command(
        "create-item",
        short_help="Convert a Sentinel3 scene into a STAC item",
    )
    @click.argument("src")
    @click.argument("dst")
    def create_item_command(src, dst):
        """Creates a STAC Collection

        Args:
            src (str): path to the scene
            dst (str): path to the STAC Item JSON file that will be created
        """
        item = create_item(src)

        item_path = os.path.join(dst, "{}.json".format(item.id))
        item.set_self_href(item_path)

        item.save_object()

        return sentinel3



import stactools.core

from stactools.sentinel3.stac import create_item

__all__ = ['create_collection', 'create_item']

stactools.core.use_fsspec()


def register_plugin(registry):
    from stactools.sentinel3 import commands
    registry.register_subcommand(commands.create_sentinel3_command)


__version__ = "0.2.0"



import pystac
from pystac import ProviderRole
from pystac.extensions.eo import Band
from pystac.link import Link

INSPIRE_METADATA_ASSET_KEY = "inspire-metadata"
SAFE_MANIFEST_ASSET_KEY = "safe-manifest"
PRODUCT_METADATA_ASSET_KEY = "product-metadata"

SENTINEL_LICENSE = Link(
    rel="license",
    target="https://sentinel.esa.int/documents/" +
    "247904/690755/Sentinel_Data_Legal_Notice",
)

SENTINEL_CONSTELLATION = "Sentinel 3"

SENTINEL_INSTRUMENTS = ["slstr"]

SENTINEL_PROVIDER = pystac.Provider(
    name="ESA",
    roles=[
        ProviderRole.PRODUCER,
        ProviderRole.PROCESSOR,
        ProviderRole.LICENSOR,
    ],
    url="https://earth.esa.int/web/guest/home",
)

SAFE_MANIFEST_ASSET_KEY = "safe-manifest"

SENTINEL_SLSTR_BANDS = {
    "S01":
    Band.create(
        name="S1",
        description="Band 1 - Cloud screening, vegetation monitoring, aerosol",
        center_wavelength=554.27,
        full_width_half_max=19.26,
    ),
    "S02":
    Band.create(
        name="S2",
        description="Band 2 - NDVI, vegetation monitoring, aerosol",
        center_wavelength=659.47,
        full_width_half_max=19.25,
    ),
    "S03":
    Band.create(
        name="S3",
        description="Band 3 - NDVI, cloud flagging, pixel co-registration",
        center_wavelength=868,
        full_width_half_max=20.6,
    ),
    "S04":
    Band.create(
        name="S4",
        description="Band 4 - Cirrus detection over land",
        center_wavelength=1374.8,
        full_width_half_max=20.8,
    ),
    "S05":
    Band.create(
        name="S5",
        description="Band 5 - Cloud clearing, ice, snow, vegetation monitoring",
        center_wavelength=1613.4,
        full_width_half_max=60.68,
    ),
    "S06":
    Band.create(
        name="S6",
        description="Band 6 - Vegetation state and cloud clearing",
        center_wavelength=2255.7,
        full_width_half_max=50.15,
    ),
    "S07":
    Band.create(
        name="S7",
        description="Band 7 - SST, LST, Active fire",
        center_wavelength=3742,
        full_width_half_max=398,
    ),
    "S08":
    Band.create(
        name="S8",
        description="Band 8 - SST, LST, Active fire",
        center_wavelength=10854,
        full_width_half_max=776,
    ),
    "S09":
    Band.create(
        name="S9",
        description="Band 9 - SST, LST",
        center_wavelength=12022.5,
        full_width_half_max=905,
    ),
    "S10":
    Band.create(
        name="F1",
        description="Band 10 - Active fire",
        center_wavelength=3742,
        full_width_half_max=398,
    ),
    "S11":
    Band.create(
        name="F2",
        description="Band 11 - Active fire",
        center_wavelength=10854,
        full_width_half_max=776,
    ),
}

SENTINEL_OLCI_BANDS = {
    "Oa01":
    Band.create(
        name="Oa01",
        description="Band 1 - Aerosol correction, "
        "improved water constituent retrieval",
        center_wavelength=400,
        full_width_half_max=15,
    ),
    "Oa02":
    Band.create(
        name="Oa02",
        description="Band 2 - Yellow substance "
        "and detrital pigments (turbidity)",
        center_wavelength=412.5,
        full_width_half_max=10,
    ),
    "Oa03":
    Band.create(
        name="Oa03",
        description="Band 3 - Chlorophyll absorption "
        "maximum, biogeochemistry, vegetation",
        center_wavelength=442.5,
        full_width_half_max=10,
    ),
    "Oa04":
    Band.create(
        name="Oa04",
        description="Band 4 - Chlorophyll",
        center_wavelength=490,
        full_width_half_max=10,
    ),
    "Oa05":
    Band.create(
        name="Oa05",
        description="Band 5 - Chlorophyll, sediment, turbidity, red tide",
        center_wavelength=510,
        full_width_half_max=10,
    ),
    "Oa06":
    Band.create(
        name="Oa06",
        description="Band 6 - Chlorophyll reference (minimum)",
        center_wavelength=560,
        full_width_half_max=10,
    ),
    "Oa07":
    Band.create(
        name="Oa07",
        description="Band 7 - Sediment loading",
        center_wavelength=620,
        full_width_half_max=10,
    ),
    "Oa08":
    Band.create(
        name="Oa08",
        description="Band 8 - 2nd Chlorophyll absorption "
        "maximum, sediment, yellow substance / vegetation",
        center_wavelength=665,
        full_width_half_max=10,
    ),
    "Oa09":
    Band.create(
        name="Oa09",
        description="Band 9 - Improved fluorescence retrieval",
        center_wavelength=673.75,
        full_width_half_max=7.5,
    ),
    "Oa10":
    Band.create(
        name="Oa10",
        description="Band 10 - Chlorophyll fluorescence peak, red edge",
        center_wavelength=681.25,
        full_width_half_max=7.5,
    ),
    "Oa11":
    Band.create(
        name="Oa11",
        description="Band 11 - Chlorophyll fluorescence "
        "baseline, red edge transition",
        center_wavelength=708.75,
        full_width_half_max=10,
    ),
    "Oa12":
    Band.create(
        name="Oa12",
        description="Band 12 - O2 absorption / clouds, vegetation",
        center_wavelength=753.75,
        full_width_half_max=7.5,
    ),
    "Oa13":
    Band.create(
        name="Oa13",
        description="Band 13 - O2 absorption / aerosol correction",
        center_wavelength=761.25,
        full_width_half_max=2.5,
    ),
    "Oa14":
    Band.create(
        name="Oa14",
        description="Band 14 - Atmospheric correction",
        center_wavelength=764.375,
        full_width_half_max=3.75,
    ),
    "Oa15":
    Band.create(
        name="Oa15",
        description="Band 15 - O2 absorption used for "
        "cloud top pressure, fluorescence over land",
        center_wavelength=767.5,
        full_width_half_max=2.5,
    ),
    "Oa16":
    Band.create(
        name="Oa16",
        description="Band 16 - Atmospheric / aerosol correction",
        center_wavelength=778.75,
        full_width_half_max=15,
    ),
    "Oa17":
    Band.create(
        name="Oa17",
        description="Band 17 - Atmospheric / aerosol "
        "correction, clouds, pixel co-registration",
        center_wavelength=865,
        full_width_half_max=20,
    ),
    "Oa18":
    Band.create(
        name="Oa18",
        description="Band 18 - Water vapour absorption reference. "
        "Common reference band with SLSTR. Vegetation monitoring",
        center_wavelength=885,
        full_width_half_max=10,
    ),
    "Oa19":
    Band.create(
        name="Oa19",
        description="Band 19 - Water vapour absorption, "
        "vegetation monitoring (maximum REFLECTANCE)",
        center_wavelength=900,
        full_width_half_max=10,
    ),
    "Oa20":
    Band.create(
        name="Oa20",
        description="Band 20 - Water vapour absorption, "
        "atmospheric / aerosol correction",
        center_wavelength=940,
        full_width_half_max=20,
    ),
    "Oa21":
    Band.create(
        name="Oa21",
        description="Band 21 - Water vapour absorption, "
        "atmospheric / aerosol correction",
        center_wavelength=1020,
        full_width_half_max=40,
    ),
}

SENTINEL_SRAL_BANDS = {
    "C":
    Band.create(
        name="C",
        description="Band C - Ionospheric correction",
        center_wavelength=5409999872.000000,  # central frequency in Hz
        full_width_half_max=290000000.000000  # bandwidth in Hz
    ),
    "Ku":
    Band.create(
        name="Ku",
        description="Band Ku - Range measurements",
        center_wavelength=13575000064.000000,  # central frequency in Hz
        full_width_half_max=320000000.000000  # bandwidth in Hz
    )
}

SENTINEL_SYNERGY_BANDS = {
    "SYN01":
    Band.create(name="SYN01",
                description="OLCI channel Oa01",
                center_wavelength=400,
                full_width_half_max=15),
    "SYN02":
    Band.create(name="SYN02",
                description="OLCI channel Oa02",
                center_wavelength=412.5,
                full_width_half_max=10),
    "SYN03":
    Band.create(name="SYN03",
                description="OLCI channel Oa03",
                center_wavelength=442.5,
                full_width_half_max=10),
    "SYN04":
    Band.create(name="SYN04",
                description="OLCI channel Oa04",
                center_wavelength=490,
                full_width_half_max=10),
    "SYN05":
    Band.create(name="SYN05",
                description="OLCI channel Oa05",
                center_wavelength=510,
                full_width_half_max=10),
    "SYN06":
    Band.create(name="SYN06",
                description="OLCI channel Oa06",
                center_wavelength=560,
                full_width_half_max=10),
    "SYN07":
    Band.create(name="SYN07",
                description="OLCI channel Oa07",
                center_wavelength=620,
                full_width_half_max=10),
    "SYN08":
    Band.create(name="SYN08",
                description="OLCI channel Oa08",
                center_wavelength=665,
                full_width_half_max=10),
    "SYN09":
    Band.create(name="SYN09",
                description="OLCI channel Oa09",
                center_wavelength=673.75,
                full_width_half_max=7.5),
    "SYN10":
    Band.create(name="SYN10",
                description="OLCI channel Oa10",
                center_wavelength=681.25,
                full_width_half_max=7.5),
    "SYN11":
    Band.create(name="SYN11",
                description="OLCI channel Oa11",
                center_wavelength=708.75,
                full_width_half_max=10),
    "SYN12":
    Band.create(name="SYN12",
                description="OLCI channel Oa12",
                center_wavelength=753.75,
                full_width_half_max=7.5),
    "SYN13":
    Band.create(name="SYN13",
                description="OLCI channel Oa16",
                center_wavelength=778.5,
                full_width_half_max=15),
    "SYN14":
    Band.create(name="SYN14",
                description="OLCI channel Oa17",
                center_wavelength=865,
                full_width_half_max=20),
    "SYN15":
    Band.create(name="SYN15",
                description="OLCI channel Oa18",
                center_wavelength=885,
                full_width_half_max=10),
    "SYN16":
    Band.create(name="SYN16",
                description="OLCI channel Oa21",
                center_wavelength=1020,
                full_width_half_max=40),
    "SYN17":
    Band.create(name="SYN17",
                description="SLSTR nadir channel S1",
                center_wavelength=555,
                full_width_half_max=20),
    "SYN18":
    Band.create(name="SYN18",
                description="SLSTR nadir channel S2",
                center_wavelength=659,
                full_width_half_max=20),
    "SYN19":
    Band.create(name="SYN19",
                description="SLSTR nadir channel S3",
                center_wavelength=865,
                full_width_half_max=20),
    "SYN20":
    Band.create(name="SYN20",
                description="SLSTR nadir channel S5",
                center_wavelength=1610,
                full_width_half_max=60),
    "SYN21":
    Band.create(name="SYN21",
                description="SLSTR nadir channel S6",
                center_wavelength=2250,
                full_width_half_max=50),
    "SYN22":
    Band.create(name="SYN22",
                description="SLSTR oblique channel S1",
                center_wavelength=555,
                full_width_half_max=20),
    "SYN23":
    Band.create(name="SYN23",
                description="SLSTR oblique channel S2",
                center_wavelength=659,
                full_width_half_max=20),
    "SYN24":
    Band.create(name="SYN24",
                description="SLSTR oblique channel S3",
                center_wavelength=865,
                full_width_half_max=20),
    "SYN25":
    Band.create(name="SYN25",
                description="SLSTR oblique channel S5",
                center_wavelength=1610,
                full_width_half_max=60),
    "SYN26":
    Band.create(name="SYN26",
                description="SLSTR oblique channel S6",
                center_wavelength=2250,
                full_width_half_max=50),
    "SYN_440":
    Band.create(name="SYN_440",
                description="OLCI channel Oa03",
                center_wavelength=442.5,
                full_width_half_max=10),
    "SYN_550":
    Band.create(name="SYN_550",
                description="SLSTR nadir and oblique channel S1",
                center_wavelength=550,
                full_width_half_max=20),
    "SYN_670":
    Band.create(name="SYN_670",
                description="SLSTR nadir and oblique channel S2",
                center_wavelength=659,
                full_width_half_max=20),
    "SYN_865":
    Band.create(
        name="SYN_865",
        description="OLCI channel Oa17, SLSTR nadir and oblique channel S2",
        center_wavelength=865,
        full_width_half_max=20),
    "SYN_1600":
    Band.create(name="SYN_1600",
                description="SLSTR nadir and oblique channel S5",
                center_wavelength=1610,
                full_width_half_max=60),
    "SYN_2250":
    Band.create(name="SYN_2250",
                description="SLSTR nadir and oblique channel S6",
                center_wavelength=2250,
                full_width_half_max=50),
    "B0":
    Band.create(name="B0",
                description="OLCI channels Oa02, Oa03",
                center_wavelength=450,
                full_width_half_max=20),
    "B2":
    Band.create(name="B2",
                description="OLCI channels Oa06, Oa07, Oa08, Oa09, Oa10",
                center_wavelength=645,
                full_width_half_max=35),
    "B3":
    Band.create(name="B3",
                description="OLCI channels Oa16, Oa17, Oa18, Oa21",
                center_wavelength=835,
                full_width_half_max=55),
    "MIR":
    Band.create(name="MIR",
                description="SLSTR nadir and oblique channels S5, S6",
                center_wavelength=1665,
                full_width_half_max=85),
}

OLCI_BANDS_TO_RESOLUTIONS = {
    "Oa01": [
        300,
    ],
    "Oa02": [
        300,
    ],
    "Oa03": [
        300,
    ],
    "Oa04": [
        300,
    ],
    "Oa05": [
        300,
    ],
    "Oa06": [
        300,
    ],
    "Oa07": [
        300,
    ],
    "Oa08": [
        300,
    ],
    "Oa09": [
        300,
    ],
    "Oa10": [
        300,
    ],
    "Oa11": [
        300,
    ],
    "Oa12": [
        300,
    ],
    "Oa13": [
        300,
    ],
    "Oa14": [
        300,
    ],
    "Oa15": [
        300,
    ],
    "Oa16": [
        300,
    ],
    "Oa17": [
        300,
    ],
    "Oa18": [
        300,
    ],
    "Oa19": [
        300,
    ],
    "Oa20": [
        300,
    ],
    "Oa21": [
        300,
    ],
}

SLSTR_BANDS_TO_RESOLUTIONS = {
    "S01": [
        500,
    ],
    "S02": [
        500,
    ],
    "S03": [
        500,
    ],
    "S04": [
        500,
    ],
    "S05": [
        500,
    ],
    "S06": [
        500,
    ],
    "S07": [
        1000,
    ],
    "S08": [
        1000,
    ],
    "S09": [
        1000,
    ],
    "S10": [
        1000,
    ],
    "S11": [
        1000,
    ],
}

SENTINEL_LICENSE = Link(
    rel="license",
    target="https://sentinel.esa.int/documents/" +
    "247904/690755/Sentinel_Data_Legal_Notice",
)



from pystac.extensions.sat import OrbitState
from stactools.core.io.xml import XmlElement


def fill_sat_properties(sat_ext, href):
    """Fills the properties for SAR.

    Based on the sat Extension.py

    Args:
        sat_ext (pystac.extensions.sat.SatExtension): The extension to be populated.
        href (str): The HREF to the scene, this is expected to be an XML file.

    Returns:
        pystac.Asset: An asset with the SAT relevant properties.
    """
    # Read meta file
    root = XmlElement.from_file(href)

    sat_ext.platform_international_designator = root.findall(
        ".//sentinel-safe:nssdcIdentifier")[0].text

    orbit_state = root.find_attr("groundTrackDirection",
                                 ".//sentinel-safe:orbitNumber")
    sat_ext.orbit_state = OrbitState(orbit_state)

    sat_ext.absolute_orbit = int(
        root.findall(".//sentinel-safe:orbitNumber")[0].text)

    sat_ext.relative_orbit = int(
        root.findall(".//sentinel-safe:relativeOrbitNumber")[0].text)


def fill_eo_properties(eo_ext, href):
    """Fills the properties for EO.

    Based on the eo Extension.py

    Args:
        eo_ext (pystac.extensions.eo.EOExtension): The extension to be populated.
        href (str): The HREF to the scene, this is expected to be an XML file.

    Returns:
        pystac.Asset: An asset with the EO relevant properties.
    """
    # Read meta file
    root = XmlElement.from_file(href)
    product_name = root.findall(".//sentinel3:productName")[0].text
    if product_name.split("_")[1] == "OL" and product_name.split(
            "_")[2] == "1":
        pass
    elif product_name.split("_")[1] == "OL" and product_name.split(
            "_")[2] == "2":
        eo_ext.cloud_cover = float(
            root.find_attr("percentage", ".//sentinel3:cloudyPixels"))
    elif product_name.split("_")[1] == "SL":
        eo_ext.cloud_cover = float(
            root.find_attr("percentage", ".//sentinel3:cloudyPixels"))
    elif product_name.split("_")[1] == "SR" and product_name.split(
            "_")[2] == "2":
        pass
    elif product_name.split("_")[1] == "SY" and product_name.split(
            "_")[2] == "2":
        eo_ext.cloud_cover = float(
            root.find_attr("percentage", ".//sentinel3:cloudyPixels"))
    else:
        raise ValueError("Unexpected value found at "
                         f"{product_name}: "
                         "this was expected to follow the sentinel 3 "
                         "naming convention, including "
                         "ending in .SEN3")


def fill_proj_properties(proj_ext, product_meta):
    """Fills the properties for PROJ.

    Based on the proj Extension.py

    Args:
        proj_ext (pystac.extensions.projection.ProjectionExtension): The extension to be populated.
        href (str): The HREF to the scene, this is expected to be an XML file.

    Returns:
        pystac.Asset: An asset with the PROJECTION relevant properties.
    """
    # Read meta file

    proj_ext.epsg = product_meta.get_epsg

    if (product_meta.scene_id.split("_")[1] == "SR"
            and product_meta.scene_id.split("_")[2] == "2"):
        pass
    elif (product_meta.scene_id.split("_")[1] == "SY"
          and product_meta.scene_id.split("_")[2] == "2"
          and product_meta.scene_id.split("_")[3] != "AOD"):
        pass
    else:
        proj_ext.shape = product_meta.get_shape
